package com.cardiff.client_project.mapper;

import com.cardiff.client_project.constant.Authority;
import com.cardiff.client_project.constant.ResponseCode;
import com.cardiff.client_project.pojo.dto.SelectDTO;
import com.cardiff.client_project.pojo.entity.CommonAdmin;
import com.cardiff.client_project.pojo.entity.Hospital;
import com.cardiff.client_project.pojo.entity.Patient;
import com.cardiff.client_project.pojo.entity.SuperUser;
import com.cardiff.client_project.pojo.vo.AdminVO;

import com.cardiff.client_project.pojo.vo.HospitalVO;
import com.cardiff.client_project.pojo.vo.SelectVO;
import com.cardiff.client_project.utils.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.simple.SimpleJdbcInsert;
import org.springframework.stereotype.Repository;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


@Repository
public class SuperAdminMapper {

    @Autowired
    JdbcTemplate jdbcTemplate;;

    /**
     * ser registration
     * @param commonAdmin
     */
    public Result insertUserInform(CommonAdmin commonAdmin) {

        //Check whether the account exists(If the user name and email address are the same, the account is confirmed to exist)
        String sql = "SELECT count(id) FROM common_admin where name = ? and email = ?";
        Integer count= jdbcTemplate.queryForObject(sql, new Object[]{commonAdmin.getName(), commonAdmin.getEmail()}, Integer.class);
        System.out.println(count);
        if(count>0){
            return Result.error(ResponseCode.ACCOUNT_EXISTS_ERROR);
        }

        //insert data
        SimpleJdbcInsert insert = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("common_admin")
                .usingGeneratedKeyColumns("id");

        Map<String, Object> parameters = new HashMap<>();
        parameters.put("name",commonAdmin.getName());
        parameters.put("password",commonAdmin.getPassword());
        parameters.put("email",commonAdmin.getEmail());
        parameters.put("phone",commonAdmin.getPhone());
        parameters.put("type",commonAdmin.getType());
        parameters.put("status",commonAdmin.getStatus());
        parameters.put("roleId",commonAdmin.getRoleId());

        Number number = insert.executeAndReturnKey(parameters);
        if(number.longValue() > 0){
            return Result.success(ResponseCode.SUCCESS);
        }else {
            return Result.error(ResponseCode.ERROR);
        }
    }

    /**
     * Batch deletion based on id
     * @param ids
     * @return
     */
    public int[] deleteByIdAndType(List<Integer> ids,String type) {
        String sql = "";
        String sql_1="";
        String sql_2="";
        if(type.equals("hospital")){
            sql = "DELETE FROM hospital WHERE id=?";
            sql_1="ALTER TABLE hospital DROP COLUMN id;";
            sql_2="ALTER TABLE hospital ADD COLUMN id INT NOT NULL PRIMARY KEY AUTO_INCREMENT FIRST;";
        }
        if(type.equals("commonAdmin")){
            sql = "DELETE FROM common_admin WHERE id=?";
            sql_1="ALTER TABLE common_admin DROP COLUMN id;";
            sql_2="ALTER TABLE common_admin ADD COLUMN id INT NOT NULL PRIMARY KEY AUTO_INCREMENT FIRST;";
        }
        List<Object[]> idList = new ArrayList<>();
        // Build parameter list
        for (Integer id : ids) {
            idList.add(new Object[]{id});
        }
        // Batch delete
        int[] item = jdbcTemplate.batchUpdate(sql, idList);
        jdbcTemplate.update(sql_1);
        jdbcTemplate.update(sql_2);
        return item;
    }

    /**
     * select admin
     * @return
     */
    public  List<AdminVO> selectAllAdmin() {

        // Parameterized queries avoid SQL injection
        String sql = "SELECT * FROM common_admin ORDER BY status DESC";
        List<AdminVO> query = jdbcTemplate.query(sql,new BeanPropertyRowMapper<>(AdminVO.class));
        return query;
    }

    /**
     * select hospital
     * @return
     */
    public List<Hospital> selectAllHospital(){

        // Parameterized queries avoid SQL injection
        String sql = "SELECT * FROM hospital ORDER BY status DESC";
        List<Hospital> query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(Hospital.class));
        System.out.println(query);
        return query;
    }


    /**
     * update data
     * @param o
     */
    public List update(Object o) {
        if(o instanceof CommonAdmin){
            CommonAdmin commonAdmin=(CommonAdmin)o;
            StringBuilder sql = new StringBuilder("UPDATE common_admin SET ");
            List<Object> params = new ArrayList<>();

            if (commonAdmin.getName() != null && !commonAdmin.getName().isEmpty()) {
                sql.append("name = ?, ");
                params.add(commonAdmin.getName());
            }

            if (commonAdmin.getEmail() != null && !commonAdmin.getEmail().isEmpty()) {
                sql.append("email = ?, ");
                params.add(commonAdmin.getEmail());
            }

            if (commonAdmin.getPhone() != null && !commonAdmin.getPhone().isEmpty()) {
                sql.append("phone = ?, ");
                params.add(commonAdmin.getPhone());
            }
            if (Integer.valueOf(commonAdmin.getStatus())!=null) {
                sql.append("status = ?, ");
                params.add(commonAdmin.getStatus());
            }
            // Remove the last comma and space
            sql.setLength(sql.length() - 2);

            // Add the WHERE clause
            sql.append(" WHERE id = ?");
            params.add(commonAdmin.getId());

            jdbcTemplate.update(sql.toString(), params.toArray());

            List<AdminVO> adminVOS = selectAllAdmin();
            return adminVOS;

        }
        if(o instanceof Hospital){

            Hospital hospital = (Hospital) o;
            System.out.println("hospital"+hospital);
            StringBuilder sql = new StringBuilder("UPDATE hospital SET ");
            List<Object> params = new ArrayList<>();

            if (hospital.getEmail() != null && !hospital.getEmail().isEmpty()) {
                sql.append("email = ?, ");
                params.add(hospital.getEmail());
            }

            if (hospital.getName() != null && !hospital.getName().isEmpty()) {
                sql.append("name = ?, ");
                params.add(hospital.getName());
            }

            if (hospital.getPhone() != null && !hospital.getPhone().isEmpty()) {
                sql.append("phone = ?, ");
                params.add(hospital.getPhone());
            }

            if (Integer.valueOf(hospital.getStatus()) != null) {
                sql.append("status = ?, ");
                params.add(hospital.getStatus());
            }

            if (hospital.getAddress() != null && !hospital.getAddress().isEmpty()) {
                sql.append("address = ?, ");
                params.add(hospital.getAddress());
            }

            if ((Integer.valueOf(hospital.getMaxAmount()) != null)){
                sql.append("maxAmount = ?, ");
                params.add(hospital.getMaxAmount());
            }

            if (Integer.valueOf(hospital.getAmountPatient()) != null) {
                sql.append("amountPatient = ?, ");
                params.add(hospital.getAmountPatient());
            }


            sql.setLength(sql.length() - 2);
            sql.append(" WHERE id = ?");
            params.add(hospital.getId());

            jdbcTemplate.update(sql.toString(), params.toArray());

            List<Hospital> hospitals = selectAllHospital();
            return hospitals;
        }
        return null;
    }



    /**
     * fuzzy query
     * @param selectDTO
     * @return
     */
    public List<SelectVO>   selectByItem(SelectDTO selectDTO) {
        System.out.println("test:"+selectDTO);
        // Get query type
        String type = selectDTO.getType();
        // Build the basic SQL and parameter list
        StringBuilder sql = new StringBuilder("SELECT * FROM");
        if(type.contains("commonAdmin")){
            sql.append(" common_admin ");
        }
        if(type.contains("hospital")){
            sql.append(" hospital ");
        }

        //Dynamic table name stitching
        sql.append(" WHERE 1=1 ");
        List<Object> params = new ArrayList<>();

        //Dynamic concatenation of query conditions
        if (selectDTO.getName() != null && selectDTO.getName()!="") {
            sql.append(" AND name LIKE ?");
            params.add("%" + selectDTO.getName() + "%");
        }
        if (selectDTO.getPhone() != null && selectDTO.getPhone()!="") {
            sql.append(" AND phone LIKE ?");
            params.add("%" + selectDTO.getPhone() + "%");
        }
        if (selectDTO.getAddress() != null && selectDTO.getAddress()!="") {
            sql.append(" AND address LIKE ?");
            params.add("%" + selectDTO.getAddress() + "%");
        }
        if (Integer.valueOf(selectDTO.getStatus())!=null) {
            sql.append(" AND status = ?, ");
            params.add(selectDTO.getStatus());
        }
        sql.setLength(sql.length() - 2);

        System.out.println("sql: "+sql.toString());
        List<SelectVO> query = jdbcTemplate.query(sql.toString(), params.toArray(), new BeanPropertyRowMapper<>(SelectVO.class));
        System.out.println("quert"+query);
        return query;

        //String type=selectVo.getType();
//        String first="select * from ";
//        String second = " where ";
//        if(selectVo.getName()!=null){
//           second+="name like '%"+selectVo.getName()+"%'";
//        }
//        if(selectVo.getPhone()!=null){
//            second+="phone like '%"+selectVo.getPhone()+"%'";
//        }
//        if(selectVo.getAddress()!=null){
//            second+="address like '%"+selectVo.getAddress()+"'";
//        }
//
//
//        List<SelectDTO> query=new ArrayList<>();
//        if("commonAdmin".equals(type)){
//            String sql=first+"common_admin"+second;
//            System.out.println(sql);
//            query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(SelectDTO.class));
//        }
//        if("hospital".equals(type)){
//            String sql=first+"hospital"+second;
//            System.out.println(sql);
//           query = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(SelectDTO.class));
//        }
//        System.out.println(query);
    }

    /**
     * Query data by name
     * @param username
     * @return
     */
    public Object getInforByName(String username){

            try {
                // 1. 查询 super_admin 表
                String sql = "select * from super_admin where name=?";
                return jdbcTemplate.queryForObject(sql, new Object[]{username}, new BeanPropertyRowMapper<>(SuperUser.class));
            } catch (Exception s) {
                try {
                    // 2. 查询 hospital 表
                    String sql = "select * from hospital where email=?";
                    return jdbcTemplate.queryForObject(sql, new Object[]{username}, new BeanPropertyRowMapper<>(Hospital.class));
                } catch (Exception h) {
                    try {
                        // 3. 查询 common_admin 表
                        String sql = "select * from common_admin where email=?";
                        return jdbcTemplate.queryForObject(sql, new Object[]{username}, new BeanPropertyRowMapper<>(CommonAdmin.class));
                    } catch (Exception c) {
                        try {
                            // 4. 查询 patient 表
                            String sql = "select * from patient where email=?";
                            return jdbcTemplate.queryForObject(sql, new Object[]{username}, new BeanPropertyRowMapper<>(Patient.class));
                        } catch (Exception p) {}
                    }
                }
            }
            // 如果所有查询都为空，返回 null
            return null;
    }

    /**
     * Get role permissions based on id
     * @param roleId
     * @return
     */
    public String getTypeById(int roleId) {
        String sql="SELECT type from role where roleId=?;";
        String type= jdbcTemplate.queryForObject(sql, new Object[]{roleId}, String.class);
        return type;
    }

    /**
     * insert hospital
     * @param hospital
     */
    public Result insertHospitalInform(Hospital hospital) {
        //Check whether the account exists(If the user name and email address are the same, the account is confirmed to exist)
        String sql = "SELECT count(id) FROM hospital where name = ? and email = ?";
        Integer count= jdbcTemplate.queryForObject(sql, new Object[]{hospital.getName(), hospital.getEmail()}, Integer.class);
        System.out.println(count);
        if(count>0){
            return Result.error(ResponseCode.ACCOUNT_EXISTS_ERROR);
        }

        //insert data
        SimpleJdbcInsert insert = new SimpleJdbcInsert(jdbcTemplate)
                .withTableName("hospital")
                .usingGeneratedKeyColumns("id");

        Map<String, Object> parameters = new HashMap<>();
        parameters.put("name",hospital.getName());
        parameters.put("password",hospital.getPassword());
        parameters.put("email",hospital.getEmail());
        parameters.put("phone",hospital.getPhone());
        parameters.put("type",hospital.getType());
        parameters.put("roleId", Authority.HOSPITAL);
        parameters.put("address",hospital.getAddress());
        parameters.put("amountPatient",hospital.getAmountPatient());
        parameters.put("maxAmount",hospital.getMaxAmount());
        parameters.put("patientId",hospital.getPatientId());
        parameters.put("deviceId",hospital.getDeviceId());
        parameters.put("status",hospital.getStatus());

        Number number = insert.executeAndReturnKey(parameters);
        if(number.longValue() > 0){
            return Result.success(ResponseCode.SUCCESS);
        }else {
            return Result.error(ResponseCode.ERROR);
        }
    }
}
